//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

// Information extacted from the shape.
//
// Wheel Sequences
//    spring#        Wheel spring motion: time 0 = wheel fully extended,
//                   the hub must be displaced, but not directly animated
//                   as it will be rotated in code.
// Other Sequences
//    steering       Wheel steering: time 0 = full right, 0.5 = center
//    breakLight     Break light, time 0 = off, 1 = breaking
//
// Wheel Nodes
//    hub#           Wheel hub, the hub must be in it's upper position
//                   from which the springs are mounted.
//
// The steering and animation sequences are optional.
// The center of the shape acts as the center of mass for the car.

//-----------------------------------------------------------------------------
datablock AudioProfile(fastImpactSound)
{
   filename    = "~/data/sound/fastimpact.WAV";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(slowImpactSound)
{
   filename    = "~/data/sound/slowimpact.wav";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(landingSound)
{
   filename    = "~/data/sound/landing.wav";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(skidSound)
{
   filename    = "~/data/sound/steam.wav";
   description = AudioDefaultLooping3d;
   preload = true;
};
datablock AudioProfile(idleSound)
{
   filename    = "~/data/sound/idle.wav";
   description = AudioDefaultLooping3d;
   preload = true;
};
datablock AudioProfile(vehiclecrash1Sound)
{
   filename    = "~/data/sound/vehiclecrash1.wav";
   description = AudioDefault3d;
   preload = true;
};

datablock ParticleData(TireParticle)
{
   textureName          = "~/data/shapes/vehicles/dustParticle";
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 0;
   colors[0]     = "0.46 0.36 0.26 1.0";
   colors[1]     = "0.46 0.46 0.36 0.0";
   sizes[0]      = 0.50;
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(TireEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 5;
   thetaMax         = 20;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "TireParticle";
};


//----------------------------------------------------------------------------

datablock WheeledVehicleTire(DefaultCarTire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = "~/data/shapes/vehicles/jeeptire.dts";
   friction = 100000; //1.5;

   // Spring that generates lateral tire forces
   lateralForce = 6000;
   lateralDamping = 4000;
   lateralRelaxation = 0.1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 6000;
   longitudinalDamping = 4000;
   longitudinalRelaxation = 0.1;
};
datablock WheeledVehicleTire(FlatCarTire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = "~/data/shapes/vehicles/jeepFlatTire.dts";
   friction = 1000;

   // Spring that generates lateral tire forces
   lateralForce = 6000;
   lateralDamping = 40;
   lateralRelaxation = 0.1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 600;
   longitudinalDamping = 40;
   longitudinalRelaxation = 1;
};
datablock WheeledVehicleSpring(FlatCarSpring)
{
   // Wheel suspension properties
   length = 0;             // Suspension travel
   force = 300;              // Spring force
   damping = 60;             // Spring damping
   antiSwayForce = 3;         // Lateral anti-sway force
};


datablock WheeledVehicleSpring(DefaultCarSpring)
{
   // Wheel suspension properties
   length = 0.3;             // Suspension travel
   force = 12000; //3000;              // Spring force
   damping = 2400; //600;             // Spring damping
   antiSwayForce = 12; //3;         // Lateral anti-sway force
};

datablock WheeledVehicleData(DefaultCar)
{
   category = "Vehicles";
   displayName = "Default car";
   shapeFile = "~/data/shapes/vehicles/jeep.dts";
   emap = true;
   nummountpoints = 4;
   minMountDist = 100;

   maxDamage = 200;
   destroyedLevel = 200;

   maxSteeringAngle = 0.785;  // Maximum steering angle, should match animation
   integration = 4;           // Force integration time: TickSec/Rate
   tireEmitter = TireEmitter; // All the tires use the same dust emitter

   // 3rd person camera settings
   cameraRoll = true;         // Roll the camera with the vehicle
   cameraMaxDist = 6;         // Far distance from vehicle
   cameraOffset = 1.5;        // Vertical offset from camera mount point
   cameraLag = 0.1;           // Velocity lag of camera
   cameraDecay = 0.75;        // Decay per sec. rate of velocity lag

   useEyePoint = false;	


	defaultTire	= defaultCarTire;
	defaultSpring	= defaultCarSpring;
	flatTire	= flatCarTire;
	flatSpring	= flatCarSpring;


   // Rigid Body
   mass = 600;
   drag = 0.6;
   bodyFriction = 1.6;
   bodyRestitution = 0.4;
   minImpactSpeed = 5;        // Impacts over this invoke the script callback
   softImpactSpeed = 5;       // Play SoftImpact Sound
   hardImpactSpeed = 15;      // Play HardImpact Sound
   groundImpactMinSpeed    = 10.0;

   // Engine
   engineTorque = 8000; //4000;       // Engine power
   engineBrake = 6000;         // Braking when throttle is 0
   brakeTorque = 50000;        // When brakes are applied
   maxWheelSpeed = 60;        // Engine scale by current speed / max speed

   // Energy
   maxEnergy = 100;
   jetForce = 3000;
   minJetEnergy = 30;
   jetEnergyDrain = 2;


   splash = PlayerSplash;
   splashVelocity = 4.0;
   splashAngle = 67.0;
   splashFreqMod = 300.0;
   splashVelEpsilon = 0.60;
   bubbleEmitTime = 0.4;
   splashEmitter[0] = PlayerFoamDropletsEmitter;
   splashEmitter[1] = PlayerFoamEmitter;
   splashEmitter[2] = PlayerBubbleEmitter;
   mediumSplashSoundVelocity = 10.0;   
   hardSplashSoundVelocity = 20.0;   
   exitSplashSoundVelocity = 5.0;

   // Sounds
//   jetSound = ScoutThrustSound;
	//engineSound = idleSound;
	//squealSound = skidSound;
	softImpactSound = slowImpactSound;
	hardImpactSound = fastImpactSound;
	wheelImpactSound = clatchSound;

//   explosion = VehicleExplosion;
	justcollided = 0;
};
datablock WheeledVehicleData(DefaultCar2)
{
   category = "Vehicles";
   displayName = " ";
  shapeFile = "~/data/shapes/buggy000/buggy.dts";
   emap = true;
   nummountpoints = 1;
   minMountDist = 5;

   maxDamage = 800;
   destroyedLevel = 800;

   maxSteeringAngle = 0.785;  // Maximum steering angle, should match animation
   integration = 4;           // Force integration time: TickSec/Rate
   tireEmitter = TireEmitter; // All the tires use the same dust emitter

   // 3rd person camera settings
   cameraRoll = true;         // Roll the camera with the vehicle
   cameraMaxDist = 15;         // Far distance from vehicle
   cameraOffset = 2.5;        // Vertical offset from camera mount point
   cameraLag = 0.0;           // Velocity lag of camera
   cameraDecay = 0.0;        // Decay per sec. rate of velocity lag

   useEyePoint = false;   


   // Rigid Body
   mass = 200;
   drag = 0.2;
   bodyFriction = 0.6;
   bodyRestitution = 0.4;
   minImpactSpeed = 5;        // Impacts over this invoke the script callback
   speedDamageScale = 10.0;

   softImpactSpeed = 5;       // Play SoftImpact Sound
   hardImpactSpeed = 15;      // Play HardImpact Sound
   groundImpactMinSpeed    = 5.0;

   // Engine
   engineTorque = 1000; //4000;       // Engine power
   engineBrake = 6000;         // Braking when throttle is 0
   brakeTorque = 0;        // When brakes are applied
   maxWheelSpeed = 60;        // Engine scale by current speed / max speed

   // Energy
   maxEnergy = 100;
   jetForce = 3000;
   minJetEnergy = 30;
   jetEnergyDrain = 2;


   // Sounds
//   jetSound = ScoutThrustSound;
   //engineSound = idleSound;
   //squealSound = skidSound;
   softImpactSound = slowImpactSound;
   hardImpactSound = fastImpactSound;
   wheelImpactSound = clatchSound;

//   explosion = VehicleExplosion;
   justcollided = 0;

  // maxinv[p51ammo] = 1000;

   forwardThrust      = 3000;
   reverseThrust      = 2000;
   lift         = 0;
   maxFowardVel      = 70;
   maxReverseVel      = 70;
   horizontalSurfaceForce   = 10;   // Horizontal center "wing" (provides "bite" into the wind for climbing/diving and turning)
   verticalSurfaceForce   = 100; 
   rollForce      = 6000;
   yawForce      = 6000;
   pitchForce      = 4000;
   rotationalDrag      = 2;
   stallSpeed      = 30;
};

//-----------------------------------------------------------------------------

function WheeledVehicleData::create(%block)
{
   %obj = new WheeledVehicle() {
      dataBlock = %block;
   };
   return(%obj);
}

//-----------------------------------------------------------------------------

function WheeledVehicleData::onAdd(%this,%obj)
{
   // Setup the car with some defaults tires & springs
   for (%i = %obj.getWheelCount() - 1; %i >= 0; %i--) {
      %obj.setWheelTire(%i,%obj.getDataBlock().defaultTire);
      %obj.setWheelSpring(%i,%obj.getDataBlock().defaultSpring);
   }
	//%obj.playaudio(0, idlesound);
	%obj.setShapeName(%obj.getDataBlock().displayName);
}



function boom(%obj, %vel, %scale)
{
     //messageAll( 'MsgClient', 'BOOM!!!');
     %obj.applyImpulse(VectorSub(%obj.getPosition(),VectorNormalize(%vel)), VectorScale(%vel, %scale));
}
function jcOff( %obj )
{
	%obj.justcollided = 0;
}
function VehicleData::onCollision(%this,%obj,%col,%vec,%speed)
{
   // Collision with other objects, including items
   

   if (%col.getclassName() $= "WheeledVehicle" && %speed > 2)
   {
	//echo("Vehicle Collsion:");
	//echo("-objPos" SPC %obj.getPosition());
	//echo("-objVel" SPC %obj.getVelocity());
	//echo("-colPos" SPC %col.getPosition());
	//echo("-colVel" SPC %col.getVelocity());
	//echo("-vec" SPC %vec);
	//echo("-speed" SPC %speed);

	//if(VectorLen(%col.getVelocity()) > VectorLen(%obj.getvelocity()) )
	//	return;
	//if (%obj.justcollided == 1)
	//	return;
	//messageall(type, 'vel %1 %vec %2', %obj.getvelocity(), %vec);
	%massratio = %this.mass / %col.getDataBlock().mass;
	%objtocol = vectorSub(%col.getPosition(), %obj.getPosition());
	%hitpos = vectorSub(%obj.getPosition(), %col.getPosition());
	//messageall(type, 'speed %1 vec %2', %speed, %vec);
	%hitpos = vectorNormalize(%hitpos);
	%hitpos = vectorScale(%objtocol, 1);
	%hitpos = vectorAdd(%hitpos, %col.getPosition());
	//%col.applyimpulse( %col.getposition(), "0 0 1000");
	//echo(vectorLen(%objtocol));
	//echo( VectorLen(%obj.getVelocity()) );
	//%col.applyimpulse( %col.getposition(), VectorScale("0 0 500", VectorLen(%obj.getVelocity()) ) );
	//%col.applyimpulse( %obj.getPosition(), VectorScale(%obj.getVelocity(), 1 * %col.getDataBlock().mass) );  //VectorScale(%vec, %speed * 20 * %massratio) );
	//%col.applyimpulse( %obj.getPosition(), VectorScale(%vec, %speed * -10 * %massratio) );
	//%obj.applyimpulse( %col.getPosition(), VectorScale(%vec, %speed * -10 / %massratio) );
	//boom(%obj, VectorScale(%obj.getVelocity(), -0.5 * %massratio), 200);

	//%col.justcollided = 1;
	//messageall(type, 'jc %1', %col.justcollided);
	//%col.schedule(500, jcOff(%col));
	//%obj.applyimpulse( %obj.getPosition(), VectorScale(%obj.getVelocity(), -10 / %massratio ) );
	
   }
	if(%col.getClassName() !$= "Item")
	{
		if(%speed > 40)
		{
			//if(%col.getClassName() $= "WheeledVehicle")
				%obj.playAudio(1, vehiclecrash1Sound);
		}
		else if(%speed > %this.hardimpactspeed)
		{
			if(%col.getClassName() $= "WheeledVehicle")
				%obj.playAudio(1, fastimpactsound);
		}
		else if(%speed > %this.softimpactspeed)
		{
			if(%col.getClassName() $= "WheeledVehicle")
				%obj.playAudio(1, slowimpactsound);
		}
	}
}   

function VehicleData::onTrigger(%this, %obj, %triggerNum, %val)
{
	if(%triggerNum == 2 && %val == 1)	//jump
	{
	echo(%obj);

		%client = %obj.getControllingClient();
		//echo("client = ", %client);
		%player = %client.player;
		%client.setControlObject(%player);
		//%obj.unMountObject(%player);
		//%player.setVelocity( VectorAdd(%player.getVelocity(), "0 10 5") );
		//%player.setTransform( VectorAdd(%player.getTransform(), "0 0 3") ); 
		//%player.playthread(0, root);		//stop sitting or whatever
	}
}


function VehicleData::onImpact(%this, %obj, %col, %vec, %speed)
{
	//echo("VehicleData::onImpact");
	%obj.damage(%obj,%obj.getPosition(),%speed * %this.speedDamageScale, "Impact");
	//%obj.damage(0, VectorAdd(%obj.getPosition(),%vec), %speed * %this.speedDamageScale, "Impact");
	//%obj.playAudio(1, slowimpactsound);
}

function VehicleData::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
	//echo("VehicleData::damage");
	%obj.setDamageLevel(%obj.getDamageLevel() + (%damage));
	//echo("car damagelevel = ", %obj.getDamageLevel());
//	if(%obj.getDamageLevel() >= %this.maxDamage)
//	{
//		%obj.setDamageState(Disabled);
//	}

//	%forward = %obj.getforwardVector();
//	%hittocenter = Vectorsub(%position, %obj.getPosition() );
//	%dot = VectorDot(%forward, %hittocenter);
//	%dot2 = VectorDot(%forward, "1 0 0");
//	%dot3 = VectorDot(%hittocenter, "1 0 0");
//	%cross = VectorCross(%obj.getforwardVector(), %hittocenter);
//	//messageall(type, 'dot %1 %2 %3', %dot, %dot2, %dot3);
//
//	%cosresult = %dot/(Vectorlen(%hittocenter)*Vectorlen(%forward));
//	messageall(type, '%1 %2', %dot2, %dot3 );
//	%theta = mAcos( %cosresult );
//	messageall(type, 'theta = %1', %theta * 180/3.1415);
//	if(%dot > 0) //front
//		echo("front");
//	else 
//		echo("back");
//	if(%dot2 > 0) //right
//	{
//		if(%dot3 > 0)
//			echo("left");
//		else
//			echo("right");
//	}
//	else
//	{
//		if(%dot3 > 0)
//			echo("right");
//		else
//			echo("left");
//	}
	
}

datablock ItemData(defaultCarSpawner)
{
	category = "Special";
	className = "Ammo";
	shapeFile = "~/data/shapes/markers/octahedron.dts";
	mass = 1;
	elasticity = 0.2;
	friction = 0.6;
	pickupRadius = 0;
	computeCRC = true;
};

function defaultCarSpawner::onAdd(%this,%obj)
{
	//echo("CARSPAWNER ADDED");
	%newcar = new WheeledVehicle() 
	{
		dataBlock = DefaultCar;
	};
	%newcar.setTransform(%obj.getTransform());
	//MissionCleanup.add(%newcar);
	//%obj.delete();
}

datablock ItemData(defaultCar2Spawner)
{
	category = "Special";
	className = "Ammo";
	shapeFile = "~/data/shapes/markers/octahedron.dts";
	mass = 1;
	elasticity = 0.2;
	friction = 0.6;
	pickupRadius = 0;
	computeCRC = true;
};

function defaultCar2Spawner::onAdd(%this,%obj)
{
	//echo("CARSPAWNER ADDED");
	%newcar = new WheeledVehicle() 
	{
		dataBlock = DefaultCar2;
	};
	%newcar.setTransform(%obj.getTransform());
	//MissionCleanup.add(%newcar);
	//%obj.delete();
}

function WheeledVehicleData::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
	//echo("WheeledVehicleData::damage");
	%obj.setDamageLevel(%obj.getDamageLevel() + (%damage));
	//echo("car state = ", %obj.getDamageState());
	//echo("WheeledVehicleData::damage 2"); 
//	if(%obj.getDamageLevel() >= %obj.getDataBlock().maxDamage)
//	{
//		if(%damage < 50)
//		{
//			%obj.setWheelTire(mfloor(getRandom() * 4),%obj.getDataBlock().flatTire);
//		}
//		else 
//		{
//			%obj.setWheelSpring(mfloor(getRandom() * 4),%obj.getDataBlock().flatSpring);
//		}
//
//		//%obj.setDamageState(Disabled);
//	}
	
	//messageall(type, 'armor Damage this = %1 obj = %2', %this, %obj);
   //if (%obj.getState() $= "Dead")
   //   return;
   //%obj.applyDamage(%damage);

//echo("WheeledVehicleData::damage 3");
//echo("WheeledVehicleData::damage 4");


//	%transform = %obj.getTransform();
//	%zvec = getWord(%transform, 5);
//	%ang = getWord(%transform,6);
//	%zang = %zvec * %ang;
//	%forward = %obj.getForwardVector();
//	%hittocenter = Vectorsub(%sourceObject.getPosition(), %obj.getPosition() );
//	%hitvector = getWord(%hittocenter, 0) @ " " @ getWord(%hittocenter, 1) @ " 0";
//	//echo(%hittocenter);
//	//echo(%hitVector);
//
//	%dot1 = VectorDot(%hittocenter, "1 0 0");
//	%cosresult1 = %dot1/(Vectorlen(%hittocenter)*Vectorlen("1 0 0"));
//	%theta1 = mAcos( %cosresult1 );
//	
//	
//	%dot2 = VectorDot(%forward, "1 0 0");
//	%cosresult2 = %dot2/(Vectorlen(%forward)*Vectorlen("1 0 0"));
//	%theta2 = mAcos( %cosresult2 );
//
//	%difference = %theta1 - %theta2;
//
//	//echo(%theta1 * (180/3.14159));
//	//echo(%theta2 * (180/3.14159));
//
//	//messageall(type, 'diff = %1', %difference * (180/3.14159) );

}

function findClosestSeat(%vehicle, %player)
{
	%datablock = %vehicle.getDataBlock();
	%node = -1;
	%dis = %dataBlock.minMountDist;
	%playerpos = %player.getPosition();
	for(%i = 0; %i < %dataBlock.numMountPoints; %i++)
	{
		if(!%vehicle.getMountNodeObject(%i))
		{
			%seatPos = getWords(%vehicle.getSlotTransform(%i), 0, 2);
			%disTemp = VectorLen(VectorSub(%seatPos, %playerPos));
			if(%disTemp <= %dis)
			{
				%node = %i;
				%dis = %disTemp;
			}
		}
	}
	return(%node);
}


function findEmptySeat(%vehicle, %player, %forceNode)
{
   %minNode = 1;
   %node = -1;
   %dataBlock = %vehicle.getDataBlock();
   %dis = %dataBlock.minMountDist;
   %playerPos = getWords(%player.getTransform(), 0, 2);
   %message = "";
   if(%dataBlock.lightOnly)
   {
      if(%player.client.armor $= "Light")
         %minNode = 0;
      else 
         %message = '\c2Only Scout Armors can pilot this vehicle.~wfx/misc/misc.error.wav';
   }
   else if(%player.client.armor $= "Light" || %player.client.armor $= "Medium")
      %minNode = 0;
   else
      %minNode = findFirstHeavyNode(%dataBlock);
   
   if(%forceNode !$= "")
      %node = %forceNode;
   else
   {
      for(%i = 0; %i < %dataBlock.numMountPoints; %i++)
         if(!%vehicle.getMountNodeObject(%i))
         {
            %seatPos = getWords(%vehicle.getSlotTransform(%i), 0, 2);
            %disTemp = VectorLen(VectorSub(%seatPos, %playerPos));
            if(%disTemp <= %dis)
            {
               %node = %i;
               %dis = %disTemp;
            }
         }
    }
   if(%node != -1 && %node < %minNode)
   {
      if(%message $= "")
      {
         if(%node == 0)
            %message = '\c2Only Scout or Assault Armors can pilot this vehicle.~wfx/misc/misc.error.wav';
         else
            %message = '\c2Only Scout or Assault Armors can use that position.~wfx/misc/misc.error.wav';
      }
      
      if(!%player.noSitMessage)
      {
         %player.noSitMessage = true;
         %player.schedule(2000, "resetSitMessage");
         messageClient(%player.client, 'MsgArmorCantMountVehicle', %message);   
      }
      %node = -1;
   }
   return %node;
}
