//---------------------------------------------
//  fence pack- deployable.  supposed to stretch between two poles
//------------------------------------------------

//exec("~/data/shapes/items/fence.cs");



datablock ItemData(fencePack)
{
	category	= "Deployable";
	className	= Pack;
	shapeFile	= "~/data/shapes/items/fencepack.dts";
	image		= fencePackImage;
	mass		= 1;
	elasticity	= 0.2;
	friction	= 0.6;
	pickupRadius	= 2;
	pickUpName	= "a fence deployable";
	computeCRC	= true;
	emap		= true;
	maxInventory	= 50;
};

datablock ShapeBaseImageData(fencePackImage)
{
	className = PackImage;
	item = fencePack;
	shapeFile = "~/data/shapes/items/fencePack.dts";
	emap = true;
	mountPoint = $BackPoint;
	offset = "0.0 -0.02 -0.1";
	scale = "0.5 0.5 0.5";
	rotation = eulertomatrix("0 0 0");
};

function fencePackImage::onUnmount(%this,%obj,%slot)
{
	%obj.pole1 = "";
}

function fencePack::onUse(%data,%obj)
{
	//messageall(type, "using fence pack");
	
	%mask =  $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::StaticShapeObjectType;
	%startPos = %obj.getEyeTransform();
	%eyeVec = %obj.getEyeVector();
	%endPos = VectorAdd(%startPos, VectorScale(%eyeVec, 5) );
	%searchResult = containerRayCast(%startPos, %endPos, %mask, %obj);

	if(%searchResult)
	{
		if(%searchResult.getDataBlock() == nameToID(deployedpole))
		{
			if(!%obj.pole1)
			{
				messageclient(%obj.client, sometype, "\c1Pole 1 set.");
				%obj.pole1 = %searchResult;
				return;
			}
			if(%obj.pole1 == %searchresult)
			{
				messageclient(%obj.client, sometype, "\c2Thats Pole 1!.");
				return;
			}
			if(!isObject(%obj.pole1) || %obj.pole1.getdatablock() != nameToID(deployedPole) )
			{
				messageclient(%obj.client, sometype, "\c2Pole 1 destroyed!");
				%obj.pole1 = %searchResult;
				messageclient(%obj.client, sometype, "\c1Pole 1 set.");
				return;
			}
			echo("deploying fence searchresult " @ %searchresult);
			messageclient(%obj.client, sometype, "\c1Deploying fence.");
			
			%pole1pos = %obj.pole1.getposition();
			%pole2pos = %searchresult.getposition();
			

			%x1 = getword(%pole1pos, 0);
			%y1 = getword(%pole1pos, 1);
			%z1 = getword(%pole1pos, 2);


			%x2 = getword(%pole2pos, 0);
			%y2 = getword(%pole2pos, 1);
			%z2 = getword(%pole2pos, 2);

			if(mAbs(%z2-%z1) > 3.6)
			{
				messageclient(%obj.client, sometype, "\c2Verticle distance to large!");
				return;
			}
			
			echo("pole1 = " @ %pole1pos);
			echo("pole2 = " @ %pole2pos);
			
			echo("x1 = " @ %x1);
			echo("x2 = " @ %x2);
			echo("y1 = " @ %y1);
			echo("y2 = " @ %y2);

			%dx = %x2 - %x1;
			%dy = %y2 - %y1;

			%vector = %dx SPC %dy SPC 0;

			echo("vector = " @ %vector);
			%length = vectorLen(%vector);
			
			%team = %obj.client.team;

			if(%length > 25)								//run too long?
			{
				messageclient(%obj.client, sometype, "\c2Poles too far apart!");
				return;
			}
			if(mFloor(%length + 0.5) + %team.count[fencePack] > %team.max[fencePack])	//will this go over the team limit?
			{
				messageclient(%obj.client, sometype, "\c2Limit of " @ %team.max[fencePack] @ "\c2 meters of fence!");
				return;
			}
			if(mFloor(%length + 0.5) > %obj.getinventory(fencePack))			//got enough fence left in your pack?
			{
				messageclient(%obj.client, sometype, "\c2You dont have enough fence left!");
				return;
			}
			else
			{
				%obj.decinventory(fencePack, mFloor(%length + 0.5));
				if(%obj.getinventory(fencePack) <= 0)
				{
					%obj.unmountimage($packslot);
				}
			}

			%theta = mAcos((%x2-%x1)/%length);



			%d = new StaticShape()
			{
				datablock = deployedfence;
			};
				
			%d.scale = mAbs(%length)/2.085 @ " 1 1";
			%d.team = %team;
			%d.playthread(0, deploy);
			%team.count[fencePack] += %length;

			//align the fence with the lower pole
			if (%z1 < %z2)
				%deployposition = %x1 SPC %y1 SPC %z1;
			else
				%deployposition = %x1 SPC %y1 SPC %z2;
			
			echo("deploypos = " @ %deployposition);

			if(%dx > 0)
			{
				if(%dy > 0)
				{
					//quad 1
					//messageall(type, "quad 1");
					%d.setTransform(%deployposition @ " 0 0 1 " @ $pi - %theta);
				}
				else
				{
					//quad 4
					//messageall(type, "quad 4");
					%d.setTransform(%deployposition @ " 0 0 1 " @ $pi + %theta);
				}
			}
			else
			{
				if(%dy > 0)
				{
					//quad 2
					//messageall(type, "quad 2");
					%d.setTransform(%deployposition @ " 0 0 1 " @ $pi - %theta );
				}
				else
				{
					//quad 3
					//messageall(type, "quad 3");
					%d.setTransform(%deployposition @ " 0 0 1 " @ $pi + %theta);
				}
			}
				

			//reset the for next deployment
			%obj.pole1 = "";
			
		}
		else
		{
			messageclient(%obj.client, sometype, "\c2 Can only deploy on poles.");
		}
	}
}



datablock StaticShapeData(deployedfence)
{
   pack = fencePack;
   className = Station;
   shapeFile = "~/data/shapes/items/fence.dts";
   maxDamage = 500;
   destroyedLevel = 500;
   disabledLevel = 400;
   explosion      = DeployablesExplosion;
      expDmgRadius = 8.0;
      expDamage = 0.35;
      expImpulse = 500.0;

   dynamicType = $TypeMasks::StationObjectType;
   isShielded = true;
   energyPerDamagePoint = 110;
   maxEnergy = 50;
   rechargeRate = 0.20;
   renderWhenDestroyed = false;
   doesRepair = true;

   deployedObject = true;

   cmdCategory = "DSupport";
   cmdIcon = CMDStationIcon;
   //cmdMiniIconName = "commander/MiniIcons/com_inventory_grey";
   targetNameTag = 'Deployable';
   targetTypeTag = 'Station';

   //debrisShapeName = "debris_generic_small.dts";
   debris = DeployableDebris;
   heatSignature = 0;
};

