//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

// Game duration in secs, no limit if the duration is set to 0
$Game::Duration = 20 * 60;

// When a client score reaches this value, the game is ended.
$Game::EndGameScore = 30;

// Pause while looking over the end game screen (in secs)
$Game::EndGamePause = 10;


//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

////////////////
//team objects//
////////////////

//the max[] values use the pack items since some things like fence
//use multiple packs but only one static object.
$Team1 = new ScriptObject()
{
	teamId = 1;
	name = Red;
	score = 0;
	numPlayers = 0;
	max[fencePack] = 250;
	max[polePack] = 30;
	max[cratePack] = 20;
	max[platPack] = 20;
	max[wallPack] = 20;
	max[foundationPack] = 20;
	count[fencePack] = 0;
	count[polePack] = 0;
	count[cratePack] = 0;
	count[platPack] = 0;
	count[wallPack] = 0;
	count[foundationPack] = 0;
}; 
$Team2 = new ScriptObject()
{
	teamId = 2;
	name = Blue;
	score = 0;
	numPlayers = 0;
	max[fencePack] = 250;
	max[polePack] = 30;
	max[cratePack] = 20;
	max[platPack] = 20;
	max[wallPack] = 20;
	max[foundationPack] = 20;
	count[fencePack] = 0;
	count[polePack] = 0;
	count[cratePack] = 0;
	count[platPack] = 0;
	count[wallPack] = 0;
	count[foundationPack] = 0;
};


function onServerCreated()
{
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "Test App";

   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "Deathmatch";

   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   exec("./constants.cs");

   exec("./audioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs"); 
   exec("./triggers.cs"); 
   exec("./inventory.cs");
   exec("./shapeBase.cs");
   exec("./item.cs");
   exec("./staticShape.cs");
   exec("./health.cs");
   exec("./weapon.cs");
   exec("./radiusDamage.cs");
   exec("./rifle.cs");
   exec("./crossbow.cs");
   exec("./player.cs");
   exec("./aiPlayer.cs");

	exec("./projectile.cs");

	//exec("./projectile.cs");
	//exec("./asdf.cs");


	exec("./car.cs");
	exec("./jeep.cs");
	exec("./tank.cs");
	exec("./p51Land.cs");
	exec("./p51Air.cs");
	exec("./p51Crash.cs");
	exec("./ball.cs");

	exec("./boat.cs");

	exec("./blood.cs");
	exec("./head.cs");

	exec("./bmg50.cs");
	exec("./cal30.cs");
	exec("./mm556.cs");
	exec("./acp45.cs");
	exec("./mm9.cs");
	exec("./pellet.cs");
	exec("./glueShot.cs");
	exec("./glueGun.cs");

	exec("./m16.cs");
	exec("./colt45.cs");	
	exec("./beretta92.cs");
	exec("./HTRifle.cs");
	exec("./knife.cs");
	exec("./rapier.cs");
	exec("./broadsword.cs");
	exec("./m1carbine.cs");
	exec("./mp5.cs");
	exec("./shotgun.cs");
	exec("./morphine.cs");

	exec("./p51gun.cs");

	exec("./reload.cs");
	
	exec("./pack.cs");
	exec("./treasurepack.cs");
	exec("./medpack.cs");
	exec("./detpack.cs");
	exec("./armorpack.cs");
	exec("./repairpack.cs");
	exec("./cratePack.cs");
	exec("./fencePack.cs");
	exec("./polePack.cs");
	exec("./platPack.cs");
	exec("./wallPack.cs");
	exec("./foundationPack.cs");
	
	exec("./fragGrenade.cs");
	exec("./thermGrenade.cs");
	exec("./gasGrenade.cs");
	exec("./flashGrenade.cs");



	exec("./hook.cs");


exec("./detpack.cs");
   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;

   exec("./cratePack.cs");
}

function onServerDestroyed()
{
   // This function is called as part of a server shutdown.
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.

   startGame();
   //echo("AFTER STARTGAME");
   //exec("fps/server/scripts/bmg50.cs");
}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed

   // Normally the game should be ended first before the next
   // mission is loaded, this is here in case loadMission has been
   // called directly.  The mission will be ended if the server
   // is destroyed, so we only need to cleanup here.
   cancel($Game::Schedule);
   $Game::Running = false;
   $Game::Cycling = false;
}


//-----------------------------------------------------------------------------

function startGame()
{
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameStart');

      // Other client specific setup..
      %cl.score = 0;
   }

   // Start the game timer
   if ($Game::Duration)
      $Game::Schedule = schedule($Game::Duration * 1000, 0, "onGameDurationEnd" );
   $Game::Running = true;
}

function endGame()
{
   if (!$Game::Running)  {
      error("endGame: No game running!");
      return;
   }

   // Stop any game timers
   cancel($Game::Schedule);

   // Inform the client the game is over
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd');
   }

   // Delete all the temporary mission objects
   resetMission();
   $Game::Running = false;
}

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.
   if ($Game::Duration && !isObject(EditorGui))
      cycleGame();
}


//-----------------------------------------------------------------------------

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      $Game::Cycling = true;
      $Game::Schedule = schedule(0, 0, "onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   $Game::Schedule = schedule($Game::EndGamePause * 1000, 0, "onCyclePauseEnd");
}

function onCyclePauseEnd()
{
   $Game::Cycling = false;

   // Just cycle through the missions for now.
   %search = $Client::MissionFileSpec;
   for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search)) {
      if (%file $= $Server::MissionFile) {
         // Get the next one, back to the first if there
         // is no next.
         %file = findNextFile(%search);
         if (%file $= "")
           %file = findFirstFile(%search);
         break;
      }
   }
   loadMission(%file);
}


//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

//function GameConnection::onClientEnterGame(%this)
//{
//   commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);
//
//   // Create a new camera object.
//   %this.camera = new Camera() {
//      dataBlock = Observer;
//   };
//   MissionCleanup.add( %this.camera );
//   %this.camera.scopeToClient(%this);
//
//   // Setup game parameters, the onConnect method currently starts
//   // everyone with a 0 score.
//   %this.score = 0;
//
//   // Create a player object.
//   %this.spawnPlayer();
//}


function GameConnection::onClientEnterGame(%this)
{
	commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);    // Create a new camera object.
	%this.camera = new Camera() 
	{
		dataBlock = Observer;
	};
	MissionCleanup.add( %this.camera );
	%this.camera.scopeToClient(%this);    // Setup game parameters, the onConnect method currently starts
	// everyone with a 0 score.
	%this.score = 0;
	%this.team = 0;  //default team is 0 (observer)    //Spawn a camera by default on a observer point
	%this.camera.setTransform(pickObserverPoint(%this));
	%this.camera.setVelocity("0 0 0");
	%this.setControlObject(%this.camera);

	//make the client send his name and appearance prefs
	commandtoclient(%this, 'updatePrefs');
}


function GameConnection::onClientLeaveGame(%this)
{
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
}


//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%this)
{
   // The control objects invoked this method when they
   // move out of the mission area.
}

function GameConnection::onEnterMissionArea(%this)
{
   // The control objects invoked this method when they
   // move back into the mission area.
}


//-----------------------------------------------------------------------------

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
{
   // Clear out the name on the corpse
   %this.player.setShapeName("");

   // Switch the client over to the death cam and unhook the player object.
   if (isObject(%this.camera) && isObject(%this.player)) {
      %this.camera.setMode("Corpse",%this.player);
      %this.setControlObject(%this.camera);
   }
   %this.player = 0;

   // Doll out points and display an appropriate message
   if (%damageType $= "Suicide" || %sourceClient == %this) {
      %this.incScore(-1);
      messageAll('MsgClientKilled','%1 takes his own life!',%this.name);
   }
   else {
      %sourceClient.incScore(1);
      messageAll('MsgClientKilled','%1 gets nailed by %2!',%this.name,%sourceClient.name);
      if (%sourceClient.score >= $Game::EndGameScore)
         cycleGame();
   }
}


//-----------------------------------------------------------------------------

function GameConnection::spawnPlayer(%this)
{
	// Combination create player and drop him somewhere

	//make the player drop pack and flag before switching so nothing dissappears
	ServerCmddropPack(%this);
	ServerCmddropFlag(%this);

	%spawnPoint = pickSpawnPoint(%this);  //This line changed
	%this.createPlayer(%spawnPoint);
	
}   


//-----------------------------------------------------------------------------

function GameConnection::createPlayer(%this, %spawnPoint)
{
   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in 
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }

   // Create the player object
   if(%this.team.name $= "Red")
   {
	%player = new Player() {
		dataBlock = femaleREDArmor;
		client = %this;
	};
   }
   else
   {
	%player = new Player() {
		dataBlock = femaleBLUEArmor;
		client = %this;
	};
   }
   %player.playthread(0, root);
   MissionCleanup.add(%player);

   // Player setup...
   %player.setTransform(%spawnPoint);
   %player.setEnergyLevel(60);
   %player.setShapeName(%this.name);
   
   // Update the camera to start with the player
   %this.camera.setTransform(%player.getEyeTransform());

   // Give the client control of the player
   %this.player = %player;
   %this.setPlayerObject(%player);
   %this.setControlObject(%player);

	//echo(%this.head);
	
	commandtoclient(%this, 'updateprefs');

	switch(%this.head)
	{
		case 1:		%player.mountImage(femaleHeadImage, $HeadSlot);
		case 2:		%player.mountImage(femaleHead2Image, $HeadSlot);
		case 3:		%player.mountImage(femaleHead3Image, $HeadSlot);
		default:	%player.mountImage(femaleHeadImage, $HeadSlot);
	}
	switch(%this.hair)
	{
		case 1:		%player.mountImage(femaleHairImage, $HairSlot);
		case 2:		%player.mountImage(femaleHair2Image, $HairSlot);
		default:	%player.mountImage(femaleHairImage, $HairSlot);
	}

   //%player.mountImage(femaleHeadImage, $HeadSlot);
   //%player.mountImage(femaleHairImage, $HairSlot);
	//echo("player created");
	//spawn equipment
	//%player.pistol = Beretta92;
	//%player.melee = Knife;
	%player.mountImage(Beretta92Packimage, $PistolSlot);
	%player.mountImage(KnifePackimage, $MeleeSlot);
	%player.incinventory(Beretta92, 1);
	%player.incinventory(Beretta92clip, 6);
	%player.incinventory(Beretta92ammo, 18);
	%player.incinventory(knife, 1);
	
	//start breathing
	//%player.playthread(2, breath);
}


//-----------------------------------------------------------------------------
// Support functions
//-----------------------------------------------------------------------------

//function pickSpawnPoint() 
//{
//   %groupName = "MissionGroup/PlayerDropPoints";
//   %group = nameToID(%groupName);
//
//   if (%group != -1) {
//      %count = %group.getCount();
//      if (%count != 0) {
//         %index = getRandom(%count-1);
//         %spawn = %group.getObject(%index);
//         return %spawn.getTransform();
//      }
//      else
//         error("No spawn points found in " @ %groupName);
//   }
//   else
//      error("Missing spawn points group " @ %groupName);
//
//   // Could be no spawn points, in which case we'll stick the
//   // player at the center of the world.
//   return "0 0 300 1 0 0 0";
//}

function pickSpawnPoint(%client)
{
	%groupName = "MissionGroup/" @ %client.team.name @ "TeamDropPoints";
	%group = nameToID(%groupName); 
	
	if (%group != -1)
	{
		%count = %group.getCount();
		if (%count != 0)
		{
			%index = getRandom(%count-1);
			%spawn = %group.getObject(%index);
			//return %spawn.getTransform();

			return getTerrainLevel(%spawn.position,%spawn.radius);
		}
		else
			error("No " @ %client.team.name @ " team spawn points found in " @ %groupName);
	}
	else
		error("Missing " @ %client.team.name @ " team spawn points group " @ %groupName);    

	// Could be no spawn points, in which case we'll stick the
	// player at the center of the world.
	return "0 0 300 1 0 0 0";
}

function getTerrainLevel(%pos,%rad)
{
	while(%retries < 500)
	{
		%x = getWord(%pos, 0) + mFloor(getRandom(%rad * 2) - %rad);
		%y = getWord(%pos, 1) + mFloor(getRandom(%rad * 2) - %rad);
		%z = getWord(%pos, 2) + mFloor(getRandom(%rad * 2) - %rad);

		%start = %x @ " " @ %y @ " 5000";
		%end = %x @ " " @ %y @ " -1";
		%ground = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);
		%z = getWord(%ground, 3); error ("Spawn Position   : " @ %x SPC %y);
		error ("Ground Level     : " @ %z); %z += 3.5;

		%position = %x @ " " @ %y @ " " @ %z;

		%mask = ($TypeMasks::VehicleObjectType |
			$TypeMasks::MoveableObjectType |
			$TypeMasks::StaticShapeObjectType |
			$TypeMasks::ForceFieldObjectType |
			$TypeMasks::InteriorObjectType |
			$TypeMasks::ItemObjectType);

		if (ContainerBoxEmpty(%mask,%position,3.5))
		{
			error ("Spawn Position Is Good");
			return %position;
		}
		else
			%retries++;
	}
	return "0 0 300 1 0 0 0";
}

/////////////
//team code//
/////////////
function pickObserverPoint(%client)
{
	%groupName = "MissionGroup/ObserverDropPoints";
	%group = nameToID(%groupName);

	if (%group != -1) 
	{
		%count = %group.getCount();
		if (%count != 0)
		{
			%index = getRandom(%count-1);
			%spawn = %group.getObject(%index);
			return %spawn.getTransform();
		}
		else
			error("No observer spawn points found in " @ %groupName);
	}
	else
		error("Missing observer spawn points group " @ %groupName);    // Could be no spawn points, in which case we'll stick the

	// player at the center of the world.
	return "0 0 300 1 0 0 0";
}

function GameConnection::joinTeam(%this, %teamid)
{
	//We only have 2 teams so if team is greater than 2 or less than 0 its invalid.
	if (%teamid > 2 || %teamid < 0)
		return false;

	//If we already are on that team return.
	if (%teamid == %this.team.teamId)
		return false;    

	%this.leaveTeam();    

	if (%teamid == 1)
		%this.team = $Team1;
	if (%teamid == 2)
		%this.team = $Team2;    MessageAll('MsgClientJoinTeam', '\c2%1 joined the %2 team',
		%this.name,
		%this.team.name,
		%this.team.teamId,
		%this,
		%this.sendGuid,
		%this.score,
		%this.isAiControlled(),
		%this.isAdmin,
		%this.isSuperAdmin);   

	%this.spawnPlayer();
}

function GameConnection::leaveTeam(%client, %teamnumber)
{
	%this.score = 0;

	messageAll('MsgClientLeaveTeam', '\c2%1 left the %2 team.',
		%client.name,
		%client.team.name,
		%client);

	if (%client.player)
		%client.player.delete();
}