//-----------------------------------------------------------------------------

// Information extacted from the shape.
//
// Wheel Sequences
//    spring#        Wheel spring motion: time 0 = wheel fully extended,
//                   the hub must be displaced, but not directly animated
//                   as it will be rotated in code.
// Other Sequences
//    steering       Wheel steering: time 0 = full right, 0.5 = center
//    breakLight     Break light, time 0 = off, 1 = breaking
//
// Wheel Nodes
//    hub#           Wheel hub, the hub must be in it's upper position
//                   from which the springs are mounted.
//
// The steering and animation sequences are optional.
// The center of the shape acts as the center of mass for the car.

//-----------------------------------------------------------------------------
datablock AudioProfile(fastImpactSound)
{
   filename    = "~/data/sound/fastimpact.WAV";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(slowImpactSound)
{
   filename    = "~/data/sound/slowimpact.wav";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(landingSound)
{
   filename    = "~/data/sound/landing.wav";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(skidSound)
{
   filename    = "~/data/sound/steam.wav";
   description = AudioDefaultLooping3d;
   preload = true;
};
datablock AudioProfile(idleSound)
{
   filename    = "~/data/sound/idle.wav";
   description = AudioDefaultLooping3d;
   preload = true;
};
datablock AudioProfile(vehiclecrash1Sound)
{
   filename    = "~/data/sound/vehiclecrash1.wav";
   description = AudioDefault3d;
   preload = true;
};

datablock ParticleData(TireParticle)
{
   textureName          = "~/data/shapes/vehicles/dustParticle";
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 0;
   colors[0]     = "0.46 0.36 0.26 1.0";
   colors[1]     = "0.46 0.46 0.36 0.0";
   sizes[0]      = 0.50;
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(TireEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 5;
   thetaMax         = 20;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "TireParticle";
};


datablock ParticleData(p51landSmokeParticle1)
{
   textureName          = "~/data/shapes/vehicles/dustParticle";
   useInvAlpha		= true;
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 500;
   colors[0]     = "0.7 0.7 0.7 1.0";
   colors[1]     = "0.9 0.9 0.9 0.0";
   sizes[0]      = 3.0;
   sizes[1]      = 8.0;
   times[0]	 = 0.0;
   times[1]	 = 1.0;
};

datablock ParticleEmitterData(p51landSmokeEmitter1)
{
   ejectionPeriodMS = 200;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 1.0;
   offsetVariance   = 1.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "p51landSmokeParticle1";
};


datablock ParticleData(p51landSmokeParticle2)
{
   textureName          = "~/data/shapes/vehicles/dustParticle";
   useInvAlpha		= true;
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 500;
   colors[0]     = "0.2 0.2 0.2 1.0";
   colors[1]     = "0.4 0.4 0.4 0.0";
   sizes[0]      = 3.0;
   sizes[1]      = 8.0;
   times[0]	 = 0.0;
   times[1]	 = 1.0;
};

datablock ParticleEmitterData(p51landSmokeEmitter2)
{
   ejectionPeriodMS = 100;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 1.0;
   offsetVariance   = 1.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "p51landSmokeParticle2";
};

datablock ParticleData(p51landSmokeParticle3)
{
   textureName          = "~/data/particles/bubbles";
   useInvAlpha		= true;
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 500;
   colors[0]     = "0.0 0.7 0.9 1.0";
   colors[1]     = "0.0 0.7 0.9 0.0";
   sizes[0]      = 3.0;
   sizes[1]      = 4.0;
   times[0]	 = 0.0;
   times[1]	 = 1.0;
};

datablock ParticleEmitterData(p51landSmokeEmitter3)
{
   ejectionPeriodMS = 100;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 1.0;
   offsetVariance   = 1.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "p51landSmokeParticle3";
};

//----------------------------------------------------------------------------

datablock WheeledVehicleTire(p51Tire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = "~/data/shapes/vehicles/p51Tire.dts";
   friction = 3; //1.5;

   // Spring that generates lateral tire forces
   lateralForce = 3000;
   lateralDamping = 2000;
   lateralRelaxation = 0.1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 3000;
   longitudinalDamping = 2000;
   longitudinalRelaxation = 0.1;
};

datablock WheeledVehicleSpring(p51Spring)
{
   // Wheel suspension properties
   length = 1.0;			 // Suspension travel
   force = 12000; //3000;		 // Spring force
   damping = 2400; //600;		 // Spring damping
   antiSwayForce = 12; //3;		 // Lateral anti-sway force
};

datablock WheeledVehicleSpring(p51TailSpring)
{
   // Wheel suspension properties
   length = 0.2;			 // Suspension travel
   force = 1200; //3000;		 // Spring force
   damping = 240; //600;		 // Spring damping
   antiSwayForce = 12; //3;		 // Lateral anti-sway force
};

datablock WheeledVehicleTire(p51FlatTire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = "~/data/shapes/vehicles/jeepFlatTire.dts";
   friction = 1000;

   // Spring that generates lateral tire forces
   lateralForce = 6000;
   lateralDamping = 40;
   lateralRelaxation = 0.1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 600;
   longitudinalDamping = 40;
   longitudinalRelaxation = 1;
};
datablock WheeledVehicleSpring(p51FlatSpring)
{
   // Wheel suspension properties
   length = 0;			// Suspension travel
   force = 300;			// Spring force
   damping = 60;		// Spring damping
   antiSwayForce = 3;		// Lateral anti-sway force
};




datablock WheeledVehicleData(p51Land)
{
   category = "Vehicles";
   displayName = " ";
   shapeFile = "~/data/shapes/vehicles/p51Land.dts";
   emap = true;
   nummountpoints = 1;
   minMountDist = 5;

   maxDamage = 800;
   destroyedLevel = 800;

   maxSteeringAngle = 0.785;  // Maximum steering angle, should match animation
   integration = 4;           // Force integration time: TickSec/Rate
   tireEmitter = TireEmitter; // All the tires use the same dust emitter

   // 3rd person camera settings
   cameraRoll = true;         // Roll the camera with the vehicle
   cameraMaxDist = 15;         // Far distance from vehicle
   cameraOffset = 2.5;        // Vertical offset from camera mount point
   cameraLag = 0.0;           // Velocity lag of camera
   cameraDecay = 0.0;        // Decay per sec. rate of velocity lag

   useEyePoint = false;	


	defaultTire	= p51Tire;
	defaultSpring	= p51Spring;
	flatTire	= p51FlatTire;
	flatSpring	= p51FlatSpring;


   // Rigid Body
   mass = 200;
   drag = 0.2;
   bodyFriction = 0.6;
   bodyRestitution = 0.4;
   minImpactSpeed = 5;        // Impacts over this invoke the script callback
   speedDamageScale = 10.0;

   softImpactSpeed = 5;       // Play SoftImpact Sound
   hardImpactSpeed = 15;      // Play HardImpact Sound
   groundImpactMinSpeed    = 5.0;

   // Engine
   engineTorque = 1000; //4000;       // Engine power
   engineBrake = 6000;         // Braking when throttle is 0
   brakeTorque = 0;        // When brakes are applied
   maxWheelSpeed = 60;        // Engine scale by current speed / max speed

   // Energy
   maxEnergy = 100;
   jetForce = 3000;
   minJetEnergy = 30;
   jetEnergyDrain = 2;


   splash = PlayerSplash;
   splashVelocity = 4.0;
   splashAngle = 67.0;
   splashFreqMod = 300.0;
   splashVelEpsilon = 0.60;
   bubbleEmitTime = 0.4;
   splashEmitter[0] = PlayerFoamDropletsEmitter;
   splashEmitter[1] = PlayerFoamEmitter;
   splashEmitter[2] = PlayerBubbleEmitter;
   mediumSplashSoundVelocity = 10.0;   
   hardSplashSoundVelocity = 20.0;   
   exitSplashSoundVelocity = 5.0;


	damageEmitter[0] = p51landSmokeEmitter1;
	damageEmitter[1] = p51landSmokeEmitter2;
	damageEmitter[2] = p51landSmokeEmitter3;
	damageEmitterOffset[0] = "0.0 -3.0 0.0 ";
	damageEmitterOffset[1] = "0.0 -3.0 0.0 ";
	damageLevelTolerance[0] = 0.3;
	damageLevelTolerance[1] = 0.7;
	numDmgEmitterAreas = 3;

   // Sounds
//   jetSound = ScoutThrustSound;
	//engineSound = idleSound;
	//squealSound = skidSound;
	softImpactSound = slowImpactSound;
	hardImpactSound = fastImpactSound;
	wheelImpactSound = clatchSound;

//   explosion = VehicleExplosion;
	justcollided = 0;

	maxinv[p51ammo] = 1000;

	forwardThrust		= 3000;
	reverseThrust		= 2000;
	lift			= 0;
	maxFowardVel		= 70;
	maxReverseVel		= 70;
	horizontalSurfaceForce	= 10;   // Horizontal center "wing" (provides "bite" into the wind for climbing/diving and turning)
	verticalSurfaceForce	= 100; 
	rollForce		= 6000;
	yawForce		= 6000;
	pitchForce		= -4000;
	rotationalDrag		= 2;
	stallSpeed		= 30;
	
};

function p51Land::onAdd(%this,%obj)
{
	parent::onAdd(%this,%obj);
	%obj.setWheelSpring(2, p51TailSpring);
	%obj.mountImage(p51gunPairImage, 0);
	%obj.mountImage(p51gunImage, 1);
}

function p51Land::onPlayerMount(%data, %obj, %player, %node)
{
	//echo("P51 MANNED!");
	//echo(%data @ " " @ %obj @ " " @ %player @ " " @ %node);
	%obj.playthread(0, ambient);
	//takeOffCheck(%obj, 500);
}

function p51Land::onStall(%this, %obj)
{
	echo("stall");
	%obj.setWheelSpring(0, p51Spring);
	%obj.setWheelSpring(1, p51Spring);
	%obj.setWheelSpring(2, p51TailSpring);
}
function p51Land::onTakeoff(%this, %obj)
{
	//echo("takeoff");
	//%obj.setWheelSpring(0, jeepFlatSpring);
	//%obj.setWheelSpring(1, jeepFlatSpring);
	//%obj.setWheelSpring(2, jeepFlatSpring);
}


function p51Land::onPlayerUnmount(%data, %obj, %player, %node)
{
	//echo("P51 UNMANNED!");
	//echo(%data @ " " @ %obj @ " " @ %player @ " " @ %node);
	%obj.stopthread(0);
}

function takeOffCheck(%vehicle, %tick)
{
	//if the vehicle is going forward, fast enough, make it take off
	echo("takeoff check");
	if(VectorLen(%vehicle.getVelocity()) > 40)
	{
		if(vectorDot(%vehicle.getVelocity(), %vehicle.getForwardVector()) > 0)	//no taking off backwards
		{
			messageAll('clientmsg', "TAKE OFF!!");
			%flyer = new FlyingVehicle()
			{
				datablock = p51air;
				initialPosition = %vehicle.getPosition();
				initialVelocity = %vehicle.getVelocity();
				sourceobject = %vehicle;
			};
			%flyer.setTransform(%vehicle.getTransform());
			%flyer.mountObject(%vehicle.getMountedObject(0), 0);
			%flyer.setDamageLevel(%vehicle.getDamageLevel());
			//%flyer.setVelocity(%vehicle.getVelocity());
			%flyer.applyImpulse( %flyer.getPosition(), vectorScale(%vehicle.getVelocity(), %flyer.getDataBlock().mass) );

			%flyer.setInventory(p51ammo, %vehicle.getInventory(p51ammo) );
			 

			schedule(%tick, 0, landCheck, %flyer, %tick);

			%vehicle.delete();

			return;
		}
	}
	//if there is someone driving, schedule next check
	if(%vehicle.getMountedObject(0))
	{
		schedule(%tick, 0, takeOffCheck, %vehicle, %tick);
	}
	
}
function p51Land::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
	//parent::damage(%this, %obj, %sourceObject, %position, %damage, %damageType);
	//echo("P51LAND DAMAGE ", %obj.getDamageLevel());
	
	%obj.setDamageLevel(%obj.getDamageLevel() + (%damage));

	if(%obj.getDamageLevel() >= %this.maxDamage)
	{	
		%newVehicle = new WheeledVehicle()
		{
			datablock = p51Crash;
		};
		%newVehicle.setTransform(%obj.getTransform());
		%newVehicle.mountObject(%obj.getMountedObject(0), 0); 
		%newVehicle.applyImpulse( %newVehicle.getPosition(), vectorScale(%obj.getVelocity(), %newVehicle.getDataBlock().mass) );
		
		%newVehicle.setInventory(p51ammo, %obj.getInventory(p51ammo) );

		//start burning the pilot
		if(%newVehicle.getMountedObject(0))
			burn(%newVehicle.getMountedObject(0), 10, 500);

		%obj.setTransform("0 0 0 0 0 1 0");
		%obj.schedule(100, delete);
	}
}


function p51Land::onDisabled(%this,%obj,%state)
{
	
}

//-----------------------------------------------------------------------------


datablock ItemData(p51Spawner)
{
	category = "Special";
	className = "Ammo";
	shapeFile = "~/data/shapes/markers/octahedron.dts";
	mass = 1;
	elasticity = 0.2;
	friction = 0.6;
	pickupRadius = 0;
	computeCRC = true;
};

function p51Spawner::onAdd(%this,%obj)
{
	%newcar = new WheeledVehicle() 
	{
		dataBlock = p51Land;
	};
	%newcar.setTransform(%obj.getTransform());
	%newcar.setInventory(p51ammo, 1000);
}

