//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

// Load dts shapes and merge animations
exec("~/data/shapes/player/player.cs");

// Timeouts for corpse deletion.
$CorpseTimeoutValue = 60 * 1000;

// Damage Rate for entering Liquid
$DamageLava       = 0.01;
$DamageHotLava    = 0.01;
$DamageCrustyLava = 0.01;

//
$PlayerDeathAnim::TorsoFrontFallForward = 1;
$PlayerDeathAnim::TorsoFrontFallBack = 2;
$PlayerDeathAnim::TorsoBackFallForward = 3;
$PlayerDeathAnim::TorsoLeftSpinDeath = 4;
$PlayerDeathAnim::TorsoRightSpinDeath = 5;
$PlayerDeathAnim::LegsLeftGimp = 6;
$PlayerDeathAnim::LegsRightGimp = 7;
$PlayerDeathAnim::TorsoBackFallForward = 8;
$PlayerDeathAnim::HeadFrontDirect = 9;
$PlayerDeathAnim::HeadBackFallForward = 10;
$PlayerDeathAnim::ExplosionBlowBack = 11;


//----------------------------------------------------------------------------
// Player Audio Profiles
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------

datablock AudioProfile(DeathCrySound)
{
   fileName = "";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(PainCrySound)
{
   fileName = "";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(ArmorMoveBubblesSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioCloseLooping3d;
   preload = true;
};

datablock AudioProfile(WaterBreathMaleSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClosestLooping3d;
   preload = true;
};


//----------------------------------------------------------------------------

datablock AudioProfile(FootLightSoftSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(FootLightHardSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(FootLightMetalSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(FootLightSnowSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(FootLightShallowSplashSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(FootLightWadingSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(FootLightUnderwaterSound)
{
   filename    = "~/data/sound/footfall.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(FootLightBubblesSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
};

//----------------------------------------------------------------------------

datablock AudioProfile(ImpactLightSoftSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
   effect = ImpactSoftEffect;
};

datablock AudioProfile(ImpactLightHardSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
   effect = ImpactHardEffect;
};

datablock AudioProfile(ImpactLightMetalSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
   effect = ImpactMetalEffect;
};

datablock AudioProfile(ImpactLightSnowSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClosest3d;
   preload = true;
   effect = ImpactSnowEffect;
};

datablock AudioProfile(ImpactLightWaterEasySound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(ImpactLightWaterMediumSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(ImpactLightWaterHardSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioDefault3d;
   preload = true;
};

datablock AudioProfile(ExitingWaterLightSound)
{
   filename    = "~/data/sound/replaceme.wav";
   description = AudioClose3d;
   preload = true;
};


//----------------------------------------------------------------------------
// Splash
//----------------------------------------------------------------------------

datablock ParticleData(PlayerSplashMist)
{
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.05;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 400;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = false;
   spinRandomMin        = -90.0;
   spinRandomMax        = 500.0;
   textureName          = "~/data/shapes/player/splash";
   colors[0]     = "0.7 0.8 1.0 1.0";
   colors[1]     = "0.7 0.8 1.0 0.5";
   colors[2]     = "0.7 0.8 1.0 0.0";
   sizes[0]      = 0.5;
   sizes[1]      = 0.5;
   sizes[2]      = 0.8;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(PlayerSplashMistEmitter)
{
   ejectionPeriodMS = 5;
   periodVarianceMS = 0;
   ejectionVelocity = 3.0;
   velocityVariance = 2.0;
   ejectionOffset   = 0.0;
   thetaMin         = 85;
   thetaMax         = 85;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   lifetimeMS       = 250;
   particles = "PlayerSplashMist";
};


datablock ParticleData(PlayerBubbleParticle)
{
   dragCoefficient      = 0.0;
   gravityCoefficient   = -0.50;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 400;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = false;
   textureName          = "~/data/shapes/player/splash";
   colors[0]     = "0.7 0.8 1.0 0.4";
   colors[1]     = "0.7 0.8 1.0 0.4";
   colors[2]     = "0.7 0.8 1.0 0.0";
   sizes[0]      = 0.1;
   sizes[1]      = 0.3;
   sizes[2]      = 0.3;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(PlayerBubbleEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 2.0;
   ejectionOffset   = 0.5;
   velocityVariance = 0.5;
   thetaMin         = 0;
   thetaMax         = 80;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "PlayerBubbleParticle";
};

datablock ParticleData(PlayerFoamParticle)
{
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.05;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 400;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = false;
   spinRandomMin        = -90.0;
   spinRandomMax        = 500.0;
   textureName          = "~/data/shapes/player/splash";
   colors[0]     = "0.7 0.8 1.0 0.20";
   colors[1]     = "0.7 0.8 1.0 0.20";
   colors[2]     = "0.7 0.8 1.0 0.00";
   sizes[0]      = 0.2;
   sizes[1]      = 0.4;
   sizes[2]      = 1.6;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(PlayerFoamEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 3.0;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 85;
   thetaMax         = 85;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "PlayerFoamParticle";
};


datablock ParticleData( PlayerFoamDropletsParticle )
{
   dragCoefficient      = 1;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = -0.0;
   lifetimeMS           = 600;
   lifetimeVarianceMS   = 0;
   textureName          = "~/data/shapes/player/splash";
   colors[0]     = "0.7 0.8 1.0 1.0";
   colors[1]     = "0.7 0.8 1.0 0.5";
   colors[2]     = "0.7 0.8 1.0 0.0";
   sizes[0]      = 0.8;
   sizes[1]      = 0.3;
   sizes[2]      = 0.0;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData( PlayerFoamDropletsEmitter )
{
   ejectionPeriodMS = 7;
   periodVarianceMS = 0;
   ejectionVelocity = 2;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 60;
   thetaMax         = 80;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = true;
   particles = "PlayerFoamDropletsParticle";
};


datablock ParticleData( PlayerSplashParticle )
{
   dragCoefficient      = 1;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = -0.0;
   lifetimeMS           = 600;
   lifetimeVarianceMS   = 0;
   colors[0]     = "0.7 0.8 1.0 1.0";
   colors[1]     = "0.7 0.8 1.0 0.5";
   colors[2]     = "0.7 0.8 1.0 0.0";
   sizes[0]      = 0.5;
   sizes[1]      = 0.5;
   sizes[2]      = 0.5;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData( PlayerSplashEmitter )
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 3;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 60;
   thetaMax         = 80;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = true;
   lifetimeMS       = 100;
   particles = "PlayerSplashParticle";
};

datablock SplashData(PlayerSplash)
{
   numSegments = 15;
   ejectionFreq = 15;
   ejectionAngle = 40;
   ringLifetime = 0.5;
   lifetimeMS = 300;
   velocity = 4.0;
   startRadius = 0.0;
   acceleration = -3.0;
   texWrap = 5.0;

   texture = "~/data/shapes/player/splash";

   emitter[0] = PlayerSplashEmitter;
   emitter[1] = PlayerSplashMistEmitter;

   colors[0] = "0.7 0.8 1.0 0.0";
   colors[1] = "0.7 0.8 1.0 0.3";
   colors[2] = "0.7 0.8 1.0 0.7";
   colors[3] = "0.7 0.8 1.0 0.0";
   times[0] = 0.0;
   times[1] = 0.4;
   times[2] = 0.8;
   times[3] = 1.0;
};


//----------------------------------------------------------------------------
// Foot puffs
//----------------------------------------------------------------------------

datablock ParticleData(LightPuff)
{
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.01;
   inheritedVelFactor   = 0.6;
   constantAcceleration = 0.0;
   lifetimeMS           = 800;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = true;
   spinRandomMin        = -35.0;
   spinRandomMax        = 35.0;
   colors[0]     = "1.0 1.0 1.0 1.0";
   colors[1]     = "1.0 1.0 1.0 0.0";
   sizes[0]      = 0.1;
   sizes[1]      = 0.8;
   times[0]      = 0.3;
   times[1]      = 1.0;
};

datablock ParticleEmitterData(LightPuffEmitter)
{
   ejectionPeriodMS = 35;
   periodVarianceMS = 10;
   ejectionVelocity = 0.2;
   velocityVariance = 0.1;
   ejectionOffset   = 0.0;
   thetaMin         = 20;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   useEmitterColors = true;
   particles = "LightPuff";
};

//----------------------------------------------------------------------------
// Liftoff dust
//----------------------------------------------------------------------------

datablock ParticleData(LiftoffDust)
{
   dragCoefficient      = 1.0;
   gravityCoefficient   = -0.01;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = true;
   spinRandomMin        = -90.0;
   spinRandomMax        = 500.0;
   colors[0]     = "1.0 1.0 1.0 1.0";
   sizes[0]      = 1.0;
   times[0]      = 1.0;
};

datablock ParticleEmitterData(LiftoffDustEmitter)
{
   ejectionPeriodMS = 5;
   periodVarianceMS = 0;
   ejectionVelocity = 2.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 90;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   useEmitterColors = true;
   particles = "LiftoffDust";
};


//----------------------------------------------------------------------------

datablock DecalData(LightMaleFootprint)
{
   sizeX       = 0.125;
   sizeY       = 0.25;
   textureName = "~/data/shapes/player/footprint";
};

datablock DebrisData( PlayerDebris )
{
   explodeOnMaxBounce = false;

   elasticity = 0.15;
   friction = 0.5;

   lifetime = 4.0;
   lifetimeVariance = 0.0;

   minSpinSpeed = 40;
   maxSpinSpeed = 600;

   numBounces = 5;
   bounceVariance = 0;

   staticOnMaxBounce = true;
   gravModifier = 1.0;

   useRadiusMass = true;
   baseRadius = 1;

   velocity = 20.0;
   velocityVariance = 12.0;
};             

datablock PlayerData(LightMaleHumanArmor)
{
   renderFirstPerson = true;
   emap = true;
   
   className = Armor;
   shapeFile = "~/data/shapes/player/female.dts";
   cameraMaxDist = 3;
   computeCRC = true;
  
   canObserve = true;
   cmdCategory = "Clients";

   cameraDefaultFov = 90.0;
   cameraMinFov = 5.0;
   cameraMaxFov = 120.0;
   
   debrisShapeName = "~/data/shapes/player/debris_player.dts";
   debris = playerDebris;

   aiAvoidThis = true;

   minLookAngle = -1.4;
   maxLookAngle = 1.4;
   maxFreelookAngle = 3.0;

   mass = 90;
   drag = 0.2;  //1.5; //for paracute
   waterDrag = 0; //10.5; //... not actualy used 
   maxdrag = 0.4;
   density = 1.1; //10;
   maxDamage = 100;
   maxEnergy =  60;
   repairRate = 0.33;
   energyPerDamagePoint = 75.0;

   rechargeRate = 0.256;

   runForce = 48 * 90;
   runEnergyDrain = 0;
   minRunEnergy = 0;
   maxForwardSpeed = 14;
   maxBackwardSpeed = 13;
   maxSideSpeed = 13;

   maxUnderwaterForwardSpeed = 8.4;
   maxUnderwaterBackwardSpeed = 7.8;
   maxUnderwaterSideSpeed = 7.8;

   jumpForce = 8.3 * 90;
   jumpEnergyDrain = 0;
   minJumpEnergy = 0;
   jumpDelay = 15;

   recoverDelay = 9;
   recoverRunForceScale = 1.2;

   minImpactSpeed = 20;
   speedDamageScale = 2.0;

   boundingBox = "1.2 1.2 2.3";
   pickupRadius = 0.75;
   
   // Damage location details
   boxNormalHeadPercentage       = 0.83;
   boxNormalTorsoPercentage      = 0.49;
   boxHeadLeftPercentage         = 0;
   boxHeadRightPercentage        = 1;
   boxHeadBackPercentage         = 0;
   boxHeadFrontPercentage        = 1;

   // Foot Prints
   decalData   = LightMaleFootprint;
   decalOffset = 0.25;
   
   footPuffEmitter = LightPuffEmitter;
   footPuffNumParts = 10;
   footPuffRadius = 0.25;

   dustEmitter = LiftoffDustEmitter;

   splash = PlayerSplash;
   splashVelocity = 4.0;
   splashAngle = 67.0;
   splashFreqMod = 300.0;
   splashVelEpsilon = 0.60;
   bubbleEmitTime = 0.4;
   splashEmitter[0] = PlayerFoamDropletsEmitter;
   splashEmitter[1] = PlayerFoamEmitter;
   splashEmitter[2] = PlayerBubbleEmitter;
   mediumSplashSoundVelocity = 10.0;   
   hardSplashSoundVelocity = 20.0;   
   exitSplashSoundVelocity = 5.0;

   // Controls over slope of runnable/jumpable surfaces
   runSurfaceAngle  = 70;
   jumpSurfaceAngle = 80;

   minJumpSpeed = 20;
   maxJumpSpeed = 30;

   horizMaxSpeed = 68;
   horizResistSpeed = 33;
   horizResistFactor = 0.35;

   upMaxSpeed = 80;
   upResistSpeed = 25;
   upResistFactor = 0.3;
   
   footstepSplashHeight = 0.35;

   //NOTE:  some sounds commented out until wav's are available

   // Footstep Sounds
   FootSoftSound        = FootLightSoftSound;
   FootHardSound        = FootLightHardSound;
   FootMetalSound       = FootLightMetalSound;
   FootSnowSound        = FootLightSnowSound;
   FootShallowSound     = FootLightShallowSplashSound;
   FootWadingSound      = FootLightWadingSound;
   FootUnderwaterSound  = FootLightUnderwaterSound;
   //FootBubblesSound     = FootLightBubblesSound;
   //movingBubblesSound   = ArmorMoveBubblesSound;
   //waterBreathSound     = WaterBreathMaleSound;

   //impactSoftSound      = ImpactLightSoftSound;
   //impactHardSound      = ImpactLightHardSound;
   //impactMetalSound     = ImpactLightMetalSound;
   //impactSnowSound      = ImpactLightSnowSound;
   
   //impactWaterEasy      = ImpactLightWaterEasySound;
   //impactWaterMedium    = ImpactLightWaterMediumSound;
   //impactWaterHard      = ImpactLightWaterHardSound;
   
   groundImpactMinSpeed    = 10.0;
   groundImpactShakeFreq   = "4.0 4.0 4.0";
   groundImpactShakeAmp    = "1.0 1.0 1.0";
   groundImpactShakeDuration = 0.8;
   groundImpactShakeFalloff = 10.0;
   
   //exitingWater         = ExitingWaterLightSound;
   
   observeParameters = "0.5 4.5 4.5";

   // Allowable Inventory Items
   maxInv[BulletAmmo] = 20;
   maxInv[HealthKit] = 1;
   maxInv[RifleAmmo] = 100;
   maxInv[CrossbowAmmo] = 50;
   maxInv[Crossbow] = 1;
   maxInv[Rifle] = 1;

	maxInv[m16ammo] = 30;
	maxInv[m16clip] = 6;
	maxInv[m16] = 1;
	maxInv[m16_noclip] = 1;
	maxInv[colt45ammo] = 7;
	maxInv[colt45clip] = 5;
	maxInv[colt45] = 1;
	maxInv[colt45_noclip] = 1;
	maxInv[beretta92ammo] = 18;
	maxInv[beretta92clip] = 5;
	maxInv[beretta92] = 1;
	maxInv[beretta92_noclip] = 1;

	maxInv[HTrifleammo] = 5;
	maxInv[HTrifleclip] = 6;
	maxInv[HTrifle] = 1;
	maxInv[HTrifle_noclip] = 1;

	maxInv[M1Carbineammo] = 15;
	maxInv[M1Carbineclip] = 6;
	maxInv[M1Carbine] = 1;
	maxInv[M1Carbine_noclip] = 1;

	maxInv[mp5ammo] = 30;
	maxInv[mp5clip] = 6;
	maxInv[mp5] = 1;
	maxInv[mp5_noclip] = 1;

	maxInv[shotgunammo] = 8;
	maxInv[shotgunclip] = 50;
	maxInv[shotgun] = 1;
	maxInv[shotgun_noclip] = 1;

	maxInv[glueGunammo] = 100;
	maxInv[glueGunclip] = 6;
	maxInv[glueGun] = 1;
	maxInv[glueGun_noclip] = 1;

	maxInv[knife] = 1;
	maxInv[rapier] = 1;
	maxInv[broadsword] = 1;
	maxInv[morphine] = 1;

	maxInv[treasurePack] = 1;
	maxInv[medPack] = 1;
	maxInv[detPack] = 1;
	maxInv[armorPack] = 1;
	maxInv[repairPack] = 1;

	maxInv[cratePack] = 5;
	maxInv[fencePack] = 30;
	maxInv[polePack] = 10;
	maxInv[platPack] = 5;
	maxInv[wallPack] = 5;
	maxInv[foundationPack] = 3;

	maxInv[dynamite] = 6;


	maxInv[fragGrenade] = 4;
	maxInv[thermGrenade] = 4;
	maxInv[gasGrenade] = 4;
	maxInv[flashGrenade] = 6;


	maxInv[hook] = 2;
};


datablock PlayerData(femaleBLUEArmor : LightMaleHumanArmor) 
{
   shapeFile = "~/data/shapes/player/female_BLUE.dts";

};

datablock PlayerData(femaleREDArmor : LightMaleHumanArmor) 
{
	shapeFile = "~/data/shapes/player/female_RED.dts";
};

datablock PlayerData(femaleVIPArmor : LightMaleHumanArmor) 
{
   shapeFile = "~/data/shapes/player/female_VIP.dts";

};

//----------------------------------------------------------------------------
// Armor Datablock methods
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------

function Armor::onAdd(%this,%obj)
{
   // Vehicle timeout
   %obj.mountVehicle = true;

   // Default dynamic armor stats
   %obj.setRechargeRate(%this.rechargeRate);
   %obj.setRepairRate(0);
}

function Armor::onRemove(%this, %obj)
{
   if (%obj.client.player == %obj)
      %obj.client.player = 0;
}

function Armor::onNewDataBlock(%this,%obj)
{
}


//----------------------------------------------------------------------------

function Armor::onMount(%this,%obj,%vehicle,%node)
{
	%vehicledata = %vehicle.getDataBlock();
	%vehicledata.onPlayerMount(%vehicle, %obj,%node);

	if (%node == 0) 
	{
		%obj.setTransform("0 0 0 0 0 1 0");
		%obj.setActionThread(%vehicle.getDatablock().mountPose[%node],true,true);
		%obj.lastWeapon = %obj.getMountedImage($WeaponSlot);

		%obj.setarmthread(head);

	//%obj.setControlObject(%vehicle);
	//%obj.client.setObjectActiveImage(%vehicle, 2);

	%obj.unmountImage($WeaponSlot);
	%obj.client.setControlObject(%vehicle);
      
	}
}

function Armor::onUnmount( %this, %obj, %vehicle, %node )
{
   //if (%node == 0)
   //   %obj.mountImage(%obj.lastWeapon, $WeaponSlot);
	if (%node == 0) 
	{
		%obj.setarmthread(look);
	}
}

function Armor::doDismount(%this, %obj, %forced)
{
	echo("ARMOR::DODISMOUNT" SPC %obj);
	//if(%obj.ismounted() && %triggerNum == 2 && %val == 1)	//jump
	//{
//		%vehicle = %obj.getObjectMount();
//		%client = %obj.client;
//		%player = %client.player;
//	
//		%vehicledata = %vehicle.getDataBlock();
//		%vehicledata.onPlayerUnMount(%vehicle, %player ,%node);
//
//		%vehicle.unMountObject(%player);
//		%player.setVelocity( VectorAdd(%player.getVelocity(), "0 10 5") );
//		%player.setTransform( VectorAdd(%player.getTransform(), "0 0 3") ); 
//		%client.setControlObject(%vehicle);
//		%client.setControlObject(%player);
//		%player.playthread(0, root);		//stop sitting or whatever
//		%player.mountTimeout = 1;
//		schedule(4000, 0, mountTimeout, %player);
	//}

	//return;

   // This function is called by player.cc when the jump trigger
   // is true while mounted
   //if (!%obj.isMounted())
   //   return;

   // Position above dismount point
   %pos    = getWords(%obj.getTransform(), 0, 2);
   %oldPos = %pos;
//   %vec[0] = " 0  0  1";
//   %vec[1] = " 0  0  1";
//   %vec[2] = " 0  0 -1";
//   %vec[3] = " 1  0  0";
//   %vec[4] = "-1  0  0";

   %vec[0] = "1  0  0";
   %vec[1] = "0  1  0";
   %vec[2] = "-1  0 0";
   %vec[3] = "0  0  1";
   %vec[4] = "0  0  -1";
   %impulseVec  = "0 0 0";
   %vec[0] = MatrixMulVector( %obj.getTransform(), %vec[0]);
	
	
   // Make sure the point is valid
   %pos = "0 0 0";
   %numAttempts = 5;
   %success     = -1;
	for (%i = 0; %i < %numAttempts; %i++) {
		%pos = VectorAdd(%oldPos, VectorScale(%vec[%i], 3));
		if(getWord(%pos, 2) < ( getTerrainHeight(getWord(%pos, 0) SPC getWord(%pos, 1)) ) )
		{
			%pos = getWord(%pos, 0) SPC getWord(%pos, 1) SPC getTerrainHeight(getWord(%pos, 0) SPC getWord(%pos, 1));
		}	
		if (%obj.checkDismountPoint(%oldPos, %pos) ) {
			%success = %i;
			%impulseVec = %vec[%i];
			break;
		}
	}
   if (%forced && %success == -1)
      %pos = %oldPos;

   //%obj.mountVehicle = false;
   //%obj.schedule(4000, "setMountVehicle", true);

	%client = %obj.client;
	%player = %client.player;
	%player.mountTimeout = 1;
	schedule(3000, 0, mountTimeout, %player);
	%vehicle = %obj.getObjectMount();

	%vehicle.unMountObject(%obj);
	%obj.unmount();
	%client.setControlObject(%vehicle);
	%client.setControlObject(%player);
	%player.playthread(0, root);		//stop sitting or whatever
	%player.setVelocity(%vehicle.getVelocity());


   // Position above dismount point
   %obj.setTransform(%pos);
   %obj.applyImpulse(%pos, VectorScale(%impulseVec, %obj.getDataBlock().mass));
   //%obj.setPilot(false);
   //%obj.vehicleTurret = "";
}


//----------------------------------------------------------------------------

function Armor::onCollision(%this,%obj,%col,%vec,%speed)
{
	//echo( "collision with ", %col.getClassName());
	//this = datablock obj = person's object id
   if (%obj.getState() $= "Dead")
      return;

   // Try and pickup all items
   if (%col.getClassName() $= "Item")
   {
      %obj.pickup(%col);
      return;
   }
      
	//push players
	if (%col.getClassName() $= "Player" ) 
	{
		//echo("hit player");
		if (%col.getState() $= "Dead")
		{
			if(1) //(%col.client.team == %obj.client.team)
			{
				//carry or ressurect your team mates
				if(%obj.isMounted() == 0 && %col.isMounted() == 0)				//if you're not in a car
				{
					//echo("hit dead guy");
					//echo(%obj.getmountedImage($WeaponSlot));
					if(%obj.getmountedImage($WeaponSlot) == nametoid(morphineimage) )
					{
						//echo("ressurection!");
						//ressurection code goes here
						%col.playthread(0, root);
						%col.setWhiteOut(1);
						%col.setDamageLevel(75);
						%col.client.setControlObject(%col);
						//cancel the respawn event, should probably keep track of this event seperateley
						%col.cancelPendingEvents();
						return;
					}
					if(!isObject(%obj.flag))	//if you dont have a "flag" object
					{
						%obj.flag = %col;
						%col.stopthread(0);
						%col.stopthread(1);
						%col.stopthread(3);
						%col.playthread(0, carried);
						%obj.mountobject(%col, $backpoint);
						%col.settransform("0 0 0 0 0 1 0");
					}
				}
			}
			else
			{
				//TODO: Loot code goes here
			}
		}
		else
		{
			%direction = VectorSub(%col.getPosition(), %obj.getPosition());
			%direction = VectorNormalize(%direction);
			%dot = VectorDot(%obj.getVelocity(), %direction);
			if( %dot > 0)
				%col.applyimpulse(%col.getPosition(),  VectorScale(%direction, 1000));
		}
	}


	
   // Mount vehicles//
   ///////////////////
   if(%col.getclassName() $= "WheeledVehicle" || %col.getclassName() $= "HoverVehicle" || %col.getclassName() $= "FlyingVehicle")	//only mount vehicles
   if (%obj.getState() !$= "Dead" && %obj.isMounted() == 0 && %speed < 5)		//dont mount dead things, things already mounted or things going real fast
   {
	//echo( "player collision with vehicle", %speed, " ", VectorLen(%obj.getvelocity()), " ", %col);
	//echo("Driver = ", %col.getMountNodeObject(1));
	if(%obj.mountTimeout == 1)
		return;
	
	%seat = findClosestSeat(%col, %obj);
	if ( %seat > -1)
	{
		%col.mountObject(%obj, %seat);
		%obj.setTransform("0 0 0 0 0 1 0");
		if(%seat == 0)
			%obj.playthread(0, sitDrive);
		else
			%obj.playthread(0, sitRide);
		return;
	}
	
   }


   %this = %col.getDataBlock();
   if ((%this.className $= WheeledVehicleData) && %obj.mountVehicle &&
         %obj.getState() $= "Move" && %col.mountable) {

      // Only mount drivers for now.
      %node = 0;
      %col.mountObject(%obj,%node);
      %obj.mVehicle = %col;
   }
}

function Armor::onImpact(%this, %obj, %collidedObject, %vec, %vecLen)
{
   %obj.damage(0, VectorAdd(%obj.getPosition(),%vec), %vecLen * %this.speedDamageScale, "Impact");
}


//----------------------------------------------------------------------------

function Armor::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
	//if you have an armor pack on, reduce damage
	if(%obj.getMountedimage($PackSlot) == nameToID(armorPackImage) )
	{
		echo("hit armor");
		%damage = %damage * 0.75;
	}

	//messageall(type, 'armor Damage this = %1 obj = %2', %this, %obj);
   if (%obj.getState() $= "Dead")
      return;
   //%obj.applyDamage(%damage);
   %obj.setDamageLevel(%obj.getDamageLevel() + (%damage));
   //blood explosion
//   %p = new ExplosionData() {
//      dataBlock        = mm556Explosion;
//      //initialVelocity  = %muzzleVelocity;
//      initialPosition  = %position;
//      sourceObject     = %obj;
//      //sourceSlot       = %slot;
//      //client           = %obj.client;
//   };
   

   %location = "Body";

   // Deal with client callbacks here because we don't have this
   // information in the onDamage or onDisable methods
   %client = %obj.client;
   %sourceClient = %sourceObject ? %sourceObject.client : 0;

   if (%obj.getState() $= "Dead")
   {
	serverCmddropPack(%obj.client);
	serverCmddropWeapon(%obj.client);
	%obj.unMount();	
	//%obj.stopthread(2);	//stop breathing
	%client.onDeath(%sourceObject, %sourceClient, %damageType, %location);
   }
}

function Armor::onDamage(%this, %obj, %delta)
{
	//messageall(type, 'armor onDamage %1!', %delta);
   // This method is invoked by the ShapeBase code whenever the 
   // object's damage level changes.
   if (%delta > 0 && %obj.getState() !$= "Dead") {

      // Increment the flash based on the amount.
      %flash = %obj.getDamageFlash() + ((%delta / %this.maxDamage) * 2);
      if (%flash > 0.75)
         %flash = 0.75;
      %obj.setDamageFlash(%flash);

      // If the pain is excessive, let's hear about it.
      if (%delta > 10)
         %obj.playPain();
   }
}

function Armor::onDisabled(%this,%obj,%state)
{
	// The player object sets the "disabled" state when damage exceeds
	// it's maxDamage value.  This is method is invoked by ShapeBase
	// state mangement code.

	// If we want to deal with the damage information that actually
	// caused this death, then we would have to move this code into
	// the script "damage" method.
	//%obj.playthread(0, root);		//??
	%obj.stopthread(0);
	%obj.stopthread(1);
	%obj.stopthread(3);
	%obj.playDeathCry();
	%obj.playDeathAnimation();
	%obj.setDamageFlash(0.75);


	// Release the main weapon trigger
	%obj.setImageTrigger(0,false);

	// Schedule corpse removal.  Just keeping the place clean.
	//%obj.schedule($CorpseTimeoutValue - 5000, "startFade", 1000, 0, true);
	//%obj.schedule($CorpseTimeoutValue, "removeArmor", %this, %obj);

	if(missionGroup.respawnTime > 0)		//0 means never respawn
	{
		schedule(missionGroup.respawnTime, 0, removeArmor, %this, %obj);
	}
	else
	{
		//if everyone on the team is dead, and there is no respawning, roundover
		%teamAlive = 0;
		for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) 
		{
			%cl = ClientGroup.getObject( %clientIndex );
			
			// Other client specific setup..
			if(%cl.team == %obj.client.team)
			{
				if(%cl.getState() !$= "Dead")
				{
					%teamAlive = 1;
					break;
				}
			}
		}
		if(!%teamAlive)
			loseRound(%obj.client.team);
	}

}
function removeArmor(%this,%obj)
{
	echo("remove armor");
	
	if(%obj.getDamageState() $= "Disabled")
	{
		%obj.client.spawnplayer();
		if(%obj.isMounted())
		{
			%obj.getObjectMount().pack = "";
		}
		if(isObject(%obj.detpack))
		{
			%obj.detpack.startfade(5000, 0, true);
			%obj.detpack.schedule(5500, delete);
		}
		%obj.delete();
	}
}	

//-----------------------------------------------------------------------------

function Armor::onLeaveMissionArea(%this, %obj)
{
   // Inform the client
   %obj.client.onLeaveMissionArea();
   echo(%obj @ " LEFT MISSION AREA");
}

function Armor::onEnterMissionArea(%this, %obj)
{
   // Inform the client
   %obj.client.onEnterMissionArea();
}

//-----------------------------------------------------------------------------

function Armor::onEnterLiquid(%this, %obj, %coverage, %type)
{
   switch(%type)
   {
      case 0: //Water
      case 1: //Ocean Water
      case 2: //River Water
      case 3: //Stagnant Water
      case 4: //Lava
         %obj.setDamageDt(%this, $DamageLava, "Lava");
      case 5: //Hot Lava
         %obj.setDamageDt(%this, $DamageHotLava, "Lava");
      case 6: //Crusty Lava
         %obj.setDamageDt(%this, $DamageCrustyLava, "Lava");
      case 7: //Quick Sand
   }
}

function Armor::onLeaveLiquid(%this, %obj, %type)
{
   %obj.clearDamageDt();
}


//-----------------------------------------------------------------------------

function Armor::onTrigger(%this, %obj, %triggerNum, %val)
{
	// This method is invoked when the player receives a trigger
	// move event.  The player automatically triggers slot 0 and
	// slot one off of triggers # 0 & 1.  Trigger # 2 is also used
	// as the jump key.

	
}


//-----------------------------------------------------------------------------
// Player methods
//-----------------------------------------------------------------------------

//----------------------------------------------------------------------------

function Player::kill(%this, %damageType)
{
   %this.damage(0, %this.getPosition(), 10000, %damageType);
}


//----------------------------------------------------------------------------

function Player::mountVehicles(%this,%bool)
{
   // If set to false, this variable disables vehicle mounting.
   %this.mountVehicle = %bool;
}

function Player::isPilot(%this)
{
   %vehicle = %this.getObjectMount();
   // There are two "if" statements to avoid a script warning.
   if (%vehicle)
      if (%vehicle.getMountNodeObject(0) == %this)
         return true;
   return false;
}


//----------------------------------------------------------------------------

function Player::playDeathAnimation(%this)
{
   //if (%this.deathIdx++ > 11)
   //   %this.deathIdx = 1;
   //%this.setActionThread("Death" @ %this.deathIdx);
   %this.playthread(0, flop);
}

function Player::playCelAnimation(%this,%anim)
{
   if (%this.getState() !$= "Dead")
      %this.setActionThread("cel"@%anim);
}


//----------------------------------------------------------------------------

function Player::playDeathCry( %this )
{
   %client = %this.client;
   //playTargetAudio( %client.target, "replaceme", AudioClosest3d, false );
}

function Player::playPain( %this )
{
   %client = %this.client;
   //playTargetAudio( %client.target, "replaceme", AudioClosest3d, false);
}
