//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

// This file contains Weapon and Ammo Class/"namespace" helper methods
// as well as hooks into the inventory system. These functions are not
// attached to a specific C++ class or datablock, but define a set of
// methods which are part of dynamic namespaces "class". The Items
// include these namespaces into their scope using the  ItemData and
// ItemImageData "className" variable.

// All ShapeBase images are mounted into one of 8 slots on a shape.
// This weapon system assumes all primary weapons are mounted into
// this specified slot:


//-----------------------------------------------------------------------------
// Weapon Class 
//-----------------------------------------------------------------------------

function Weapon::onUse(%data,%obj)
{
   // Default behavoir for all weapons is to mount it into the
   // this object's weapon slot, which is currently assumed
   // to be slot 0
   if (%obj.getMountedImage($WeaponSlot) != %data.image.getId()) {
      %obj.mountImage(%data.image, $WeaponSlot);
      if (%obj.client)
         messageClient(%obj.client, 'MsgWeaponUsed', '\c0Weapon selected');
   }
}

function Weapon::onPickup(%this, %obj, %shape, %amount)
{
	
   // The parent Item method performs the actual pickup.
   // For player's we automatically use the weapon if the
   // player does not already have one in hand.
	%ammoamount = %obj.ammo;			//it loses this info on the parent pickup
	if(%obj.isStatic())
	{
		%ammoamount = 9999;
	}
	%ammotype = %obj.getDataBlock().image.ammo;	//it loses this info on the parent pickup
   if (Parent::onPickup(%this, %obj, %shape, %amount)) {

	//messageClient(%shape.client, sometype,  'Picked up %1', %this.pickUpName);
	//update the player's weapon buttons
	%slot = %this.slot;
	if(%slot == $PrimarySlot)
		%this.primary = %this;
	if(%slot == $SecondarySlot)
	{
		if(!(%this.primary))
			%this.primary = %this;
		else
			%this.secondary = %this;
	}
	if(%slot == $PistolSlot)
		%this.pistol = %this;
	if(%slot == $GrenadeSlot)
		%this.grenade = %this;
	if(%slot == $MeleeSlot)
		%this.melee = %this;


	
	//give them the ammo thats in the gun
	%shape.incinventory(%ammotype, %ammoamount);

	//use it or store it?
      if (%shape.getClassName() $= "Player" && %shape.getMountedImage($WeaponSlot) == 0)  
      {
         %shape.use(%this);
      }
      else if (%shape.getClassName() $= "Player" && %shape.getMountedImage($WeaponSlot) != 0)
      {
		//mount show image?
		//%this is an item
		%item = nameToID(%this);
		%player = %shape;
		if (%player.primary == %item)
			%player.mountImage(%item.packimage, $PrimarySlot);	
		else if (%player.secondary == %item)
			%player.mountImage(%item.packimage, $SecondarySlot);
		else if (%player.pistol == %item)
			%player.mountImage(%item.packimage, $PistolSlot);
		else if (%player.grenade == %item)
			%player.mountImage(%item.packimage, $GrenadeSlot);
		else if (%player.melee == %item)
			%player.mountImage(%item.packimage, $MeleeSlot);
		//%shape.mountImage(%this.packimage, %this.slot);
      }
   }
}

function Weapon::onInventory(%this,%obj,%amount)
{
   // Weapon inventory has changed, make sure there are no weapons
   // of this type mounted if there are none left in inventory.
   if (!%amount && (%slot = %obj.getMountSlot(%this.image)) != -1)
      %obj.unmountImage(%slot);
}   


//-----------------------------------------------------------------------------
// Weapon Image Class
//-----------------------------------------------------------------------------

//old
//function WeaponImage::onMount(%this,%obj,%slot)
//{
//   // Images assume a false ammo state on load.  We need to
//   // set the state according to the current inventory.
//   if (%obj.getInventory(%this.ammo))
//      %obj.setImageAmmo(%slot,true);
//}

function WeaponImage::onMount(%this,%obj,%slot)
{
	// Images assume a false ammo state on load.  We need to
	// set the state according to the current inventory.
	if (%obj.getInventory(%this.ammo)) {   %obj.setImageAmmo(%slot,true);
	  %currentAmmo = %obj.getInventory(%this.ammo); } else {
	  %currentAmmo = 0;
	} %obj.client.setAmmoAmountHud(%currentAmmo);
	

	//%obj.playThread(0, %this.rootThread);
	%obj.setArmThread(%this.armThread);
}

function WeaponImage::onUnMount(%this,%obj,%slot)
{
	//if you still have the item (didnt drop it), mount the item's "packimage"
	%item = %this.item;
	if ( %obj.getInventory(%item) )
	{
		//%packimage = %item.packimage;
		//messageall("clientmsg", 'mounting image: %1', %packimage);
		//%obj.mountImage(%item.packimage, %item.slot);
		%item = nameToID(%item);
		if (%obj.primary == %item)
			%obj.mountImage(%item.packimage, $PrimarySlot);	
		else if (%obj.secondary == %item)
			%obj.mountImage(%item.packimage, $SecondarySlot);
		else if (%obj.pistol == %item)
			%obj.mountImage(%item.packimage, $PistolSlot);
		else if (%obj.grenade == %item)
			%obj.mountImage(%item.packimage, $GrenadeSlot);
		else if (%obj.melee == %item)
			%obj.mountImage(%item.packimage, $MeleeSlot);
	}			
	%obj.recoil = 0;
}

function WeaponImage::onFire(%this, %obj, %slot)
{
	%projectile = %this.projectile;
	
	// Decrement inventory ammo. The image's ammo state is updated
	// automatically by the ammo inventory hooks.
	if(%this.ammo !$= "")					//some guns do not use ammo
		%obj.decInventory(%this.ammo,1);

	

	// Determine initial projectile velocity based on the 
	// gun's muzzle point and the object's current velocity
	%muzzleVector = %obj.getMuzzleVector(%slot);
	%acc = 0; //%this.accuracy + %this.accuracyStep * %obj.recoil;
	if(%acc > %this.maxAccuracy)
		%acc = %this.maxAccuracy;
	%xrand = (getRandom() * %acc) - (%acc/2);
	%yrand = (getRandom() * %acc) - (%acc/2);
	%zrand = (getRandom() * %acc) - (%acc/2);

	%muzzleVector = VectorAdd(%muzzleVector, %xrand @ " " @ %yrand @ " " @ %zrand);
	%muzzleVector = VectorNormalize(%muzzleVector);
	%objectVelocity = %obj.getVelocity();
	%muzzleVelocity = VectorAdd(
	  VectorScale(%muzzleVector, %projectile.muzzleVelocity),
	  VectorScale(%objectVelocity, %projectile.velInheritFactor));

	%obj.recoil = %obj.recoil + 1;				//increment players recoil

	// Create the projectile object

	%p = new (%this.projectileType)() 
	{
		dataBlock        = %projectile;
		initialVelocity  = %muzzleVelocity;
		initialPosition  = %obj.getMuzzlePoint(%slot);
		sourceObject     = %obj;
		sourceSlot       = %slot;
		client           = %obj.client;
	};
	MissionCleanup.add(%p);
	return %p;


}
function WeaponImage::onStopFire(%this, %obj, %slot)
{
	%obj.recoil = 0;
}
function WeaponImage::changeClip(%data, %obj, %slot)
{
	%ammo = %data.ammo;
	%clip = %data.clip;
	%item = %data.item;
	%emptyItem = %item.emptyItem;
	%emptyImage = %emptyItem.image;

echo("CHANGECLIP:" SPC %ammo SPC %clip SPC %item SPC %emptyitem SPC %emptyimage);

	if(%obj.getinventory(%ammo) == %obj.getDatablock().maxInv[%ammo] )	//got full ammo?
	{
		messageclient(%obj.client, sometype, '\c2Magazine already full');
		return;
	}
	if(%obj.getinventory(%clip) )					//got any clips left?
	{
		%obj.decInventory(%item, 1);				//take away 
		%obj.incInventory(%emptyItem, 1);			//give noclip 
		//throw clip debris here??
		%obj.unmountImage(%slot);				//unnessesary? 
		%obj.mountImage(%emptyImage, %slot);			//dont forget that you mount IMAGES 
	}
	else
	{
		if(%obj.getinventory(%ammo) == 0 )			//empty mag? toss it.
		{
			%obj.decInventory(%item, 1);			//take away 
			%obj.incInventory(%emptyItem, 1);		//give noclip 
			//throw clip debris here??
			%obj.unmountImage(%slot);			//unnessesary? 
			%obj.mountImage(%emptyImage, %slot);		//dont forget that you mount IMAGES 	
		}
		else
			messageclient(%obj.client, sometype, "\c2You're on your last clip...");
		return;
	}	
}

function WeaponImage::onAddClip(%data, %obj, %slot)
{	
	%ammo = %data.ammo;
	%clip = %data.clip;
	%item = %data.item;
	%emptyItem = %item.emptyItem;
	%emptyImage = %emptyItem.image;
	%fullItem = %emptyItem.fullItem;
	%fullImage = %fullItem.image;
	//if they have a clip
	if( %obj.getInventory(%clip) )
	{	
		%obj.decInventory(%emptyItem, 1);	//take the player's noclip
		%obj.incInventory(%fullItem, 1);	//give them a regular 
		%obj.decInventory(%clip, 1);		//take away 1  clip
		%obj.incInventory(%ammo, 400);		//give them  rounds

		//fix mounts
		%obj.unmountImage(%slot);
		%obj.mountImage(%fullImage, %slot);	//hack
	}
	else
		messageClient( %obj.client, 'MsgCantUsePack', '\c2No more clips!~wfx/misc/misc.error.wav' );
}
//-----------------------------------------------------------------------------
// Ammmo Class
//-----------------------------------------------------------------------------

function Ammo::onInventory(%this,%obj,%amount)
{
   // The ammo inventory state has changed, we need to update any
   // mounted images using this ammo to reflect the new state.
   for (%i = 0; %i < 8; %i++) {
      if ((%image = %obj.getMountedImage(%i)) > 0)
         if (isObject(%image.ammo) && %image.ammo.getId() == %this.getId())
            %obj.setImageAmmo(%i,%amount != 0);
   }
}   

//-----------------------------------------------------------------------------
//Ammo Display
//----------------------------------------------------------------------------- 
function GameConnection::setAmmoAmountHud(%client, %amount)
{
    commandToClient(%client, 'SetAmmoAmountHud', %amount);
}